class SVGTree {
  svgElem = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
  cardWidth  = 120;
  cardHeight = 310;

  constructor(parent, id) {
    //this.svgElem.setAttributeNS('http://www.w3.org/1999/xlink', 'xlink:href', '#down-arrow');
    this.svgElem.setAttribute('xmlns', "http://www.w3.org/2000/svg");
    this.svgElem.style.width  = "100%";
    this.svgElem.style.height = "100%";
    this.svgElem.id = id;
    this.svgElem.innerHTML = `
    <defs>
      <linearGradient id="gradient-card-bg" x1="0%" y1="0%" x2="0%" y2="100%"> 
        <stop offset="0%" stop-color="#edf7ff" /> 
        <stop offset="100%" stop-color="#cde7ee" /> 
      </linearGradient>
    </defs>
    <style>
    text {
      font-family:arial,helvetica;  
      font-size: 0.8em;
      overflow: hidden;
      text-overflow: ellipsis;
    }
    .name {
      font-weight: bold;
    }
    .image {
      object-fit: cover;
      object-position: center;
      overflow: hidden;
    }
    .comment {
      font-family: Courier;
    }
    .dashes {
      stroke-dasharray: 3,3;
      stroke: blue;
      stroke-width: 2;
      fill: none;
    }
    .card {
      cursor: pointer;
    }
    .line {
      stroke-width: 10;
      fill: transparent;
      opacity: 70%;
    }
    .link_lines {
      stroke-width: 120;
      stroke-linecap: round;
      fill: transparent;
      display: none;
    }
    </style>
    `;
    parent.appendChild(this.svgElem);
  }

  drawCard(person, groupColor, linkColor) {
    const cardDouble = person.partner != null;
    let node = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
    node.setAttribute('x', person.x);
    node.setAttribute('y', person.y);
    const width = (cardDouble ? this.cardWidth * 2 : this.cardWidth);
    node.setAttribute('width',  width);
    node.setAttribute('height', this.cardHeight);
    node.setAttribute('viewBox', `0 0 ${width} ${this.cardHeight}`);
    node.style.overflow = "visible";
    node.id = `card_${person.id}`;
    node.classList.add('card');
    const name = wordWrap(html2svg(person.name || ""), 17, 3);
    const partner = wordWrap(html2svg(person.partner ? person.partner.name : ""), 18, 3);
    let comment = wordWrap(html2svg(person.comment || ""), cardDouble ? 30 : 15, 3);
    if(!person.comment && person.partner && !person.partner.hasCard)
      comment = wordWrap(html2svg(person.partner.comment || ""), cardDouble ? 30 : 15, 3);
      
    let html = `
    <rect x="0" y="0" rx="10" ry="10" width="${width}" height="${this.cardHeight}"
      style="fill:${linkColor === null ? 'url(#gradient-card-bg)' : linkColor};stroke:${groupColor};stroke-width:2;opacity:0.5" />    
    `;
    if(person.image) {
      html += `
      <clipPath id="clip_${person.id}">
        <rect id="rect" rx="6" x="10" y="45" width="${width-20}" height="150"/>
      </clipPath>
      <a xlink:href="https://pi.justprojects.de/stammbaum/person/${person.id}/image/" target="_blank">
        <image x="10" y="45" height="150" id="image_${person.id}" class="image" xlink:href="" clip-path="url(#clip_${person.id})" preserveAspectRatio="xMinYMin slice"/>
      </a>
      `;
      fetch(`/stammbaum/person/${person.id}/thumb/`).then(function(response) {
        return response.blob();
      }).then(function(myBlob) {
        var reader = new FileReader();
        reader.onloadend = function() {
          document.getElementById(`image_${person.id}`).setAttribute('xlink:href', reader.result);
        }
        reader.readAsDataURL(myBlob);
        var objectURL = URL.createObjectURL(myBlob);
      });
    }
    
    html += `
    <text class="name" text-anchor="middle" y="20">
      <tspan x="${this.cardWidth*.5}" dy="0">${name[0]}</tspan>
      <tspan x="${this.cardWidth*.5}" dy="1.2em">${name[1]}</tspan>
      <tspan x="${this.cardWidth*.5}" dy="1.2em">${name[2]}</tspan>
    </text>
    <text y="210" text-anchor="middle" class="birth">
      <tspan class="birth" dy="0" x="${this.cardWidth*.5}">${person.birth_date ? '*' + formatDate(person.birth_date) : ''}</tspan>
      <tspan class="birth-loc" dy="1.2em" x="${this.cardWidth*.5}">${person.birth_town || ''}</tspan>
      <tspan class="death" dy="1.2em" x="${this.cardWidth*.5}">${person.death_date ? '&dagger;' + formatDate(person.death_date) : ''}</tspan>
      <tspan class="death-loc" dy="1.2em" x="${this.cardWidth*.5}">${person.death_town || ''}</tspan>
    </text>
    <text y="272">
      <tspan class="comment" dy="0" x="10">${comment[0]}</tspan>
      <tspan class="comment" dy="1.2em" x="10">${comment[1]}</tspan>
      <tspan class="comment" dy="1.2em" x="10">${comment[2]}</tspan>
    </text>
    `;

    if(cardDouble) {
      html = `<path d="M ${this.cardWidth-1} 0 L ${this.cardWidth-1} 260" class="dashes"/>` + html + `
      <text class="name partner" text-anchor="middle" y="20">
        <tspan x="${this.cardWidth*1.5}" dy="0">${partner[0]}</tspan>
        <tspan x="${this.cardWidth*1.5}" dy="1.2em">${partner[1]}</tspan>
        <tspan x="${this.cardWidth*1.5}" dy="1.2em">${partner[2]}</tspan>
      </text>
      <text y="210" text-anchor="middle" class="birth">
        <tspan class="birth" dy="0" x="${this.cardWidth*1.5}">${person.partner.birth_date ? '*' + formatDate(person.partner.birth_date) : ''}</tspan>
        <tspan class="birth-loc" dy="1.2em" x="${this.cardWidth * 1.5}">${person.partner.birth_town || ''}</tspan>
        <tspan class="death" dy="1.2em" x="${this.cardWidth*1.5}">${person.partner.death_date ? '&dagger;' + formatDate(person.partner.death_date) : ''}</tspan>
        <tspan class="death-loc" dy="1.2em" x="${this.cardWidth * 1.5}">${person.partner.death_town || ''}</tspan>
      </text>
      `;
    }

    node.innerHTML = html;
    if(document.getElementsByClassName('svg-pan-zoom_viewport').length > 0)
      this.svgElem.firstChild.appendChild(node);
    else
      this.svgElem.appendChild(node);
    return node;
  }

  drawLines(person, children, groupColor) {
    let path = document.createElementNS('http://www.w3.org/2000/svg', 'path'), d = '';
    const cardDouble = person.partner != null;
    
    // find the highest common mid y level
    const upperRadius = children.reduce((min, child) => Math.min(min, Math.abs(person.y + this.cardHeight - child.y) / 2), 1000);

    for(const child of children) {
      const x1 = person.x + this.cardWidth * (cardDouble ? 1.0 : 0.5);
      const y1 = person.y + this.cardHeight;
      const x2 = child.x + this.cardWidth * 0.5;
      const y2 = child.y;
      const ym = (y1 + y2) / 2;
      const radius = Math.abs(y1 - y2) * 0.5;
      const xd = Math.abs(x1 - x2);
      const dir = (x1 < x2 ? 1 : -1);

      d += ` M${x1} ${y1}`; //move to start

      if(xd <= radius + 5) {
        d += ` C${x1} ${ym}, ${x2} ${ym}, ${x2} ${y2}`; // cubic bezier
      } else {
        d += ` q0 ${upperRadius}, ${dir*upperRadius} ${upperRadius}`; // 90° quadratic bezier

        if(xd <= 2*radius) {
          d += ` C${x1 + upperRadius*dir} ${ym}, ${x2} ${ym}, ${x2} ${y2}`; // cubic bezier
        } else {
          d += ` L${x2 - radius*dir} ${y1+upperRadius}`; // line side
          d += ` q${dir*radius} 0, ${dir*radius} ${radius}`; // 90° quadratic bezier
          d += ` L${x2} ${y2}`; // line down
        }
      }
    }
    path.setAttribute('d', d);
    path.setAttribute('stroke', groupColor);
    path.classList.add('line');
    path.id = `line_${person.id}`;
    if(document.getElementsByClassName('svg-pan-zoom_viewport').length > 0)
      this.svgElem.firstChild.appendChild(path);
    else
      this.svgElem.appendChild(path);
    return path;
  }

  drawLink(link) {
    let path = document.createElementNS('http://www.w3.org/2000/svg', 'path'), d = '';
    const x1 = link.from_person.x + this.cardWidth;
    const y1 = link.from_person.y + this.cardHeight / 2;
    const x2 = link.to_person.x + this.cardWidth;
    const y2 = link.to_person.y + this.cardHeight / 2;
    const ym = (y1 + y2) / 2;
    d += ` M${x1} ${y1}`;
    d += ` C${x1} ${ym} ${x2} ${ym} ${x2} ${y2}`;

    path.setAttribute('d', d);
    path.setAttribute('stroke', link.from_person.color);
    path.classList.add('link_lines');
    const minId = Math.min(link.from_person_id, link.to_person_id);
    const maxId = Math.max(link.from_person_id, link.to_person_id);
    path.id = `link_line_${minId}_${maxId}`;
    if(document.getElementsByClassName('svg-pan-zoom_viewport').length > 0)
      this.svgElem.firstChild.appendChild(path);
    else
      this.svgElem.appendChild(path);
    return path;
  }
}

function wordWrap(str = "", maxWidth, lines) {
  let res = new Array(lines).fill("");
  if(typeof str != "string")
    return res;
  
  let parts = str.split(' ');
  for(let i=0; i<lines; i++) {
    while(parts.length > 0 && res[i].length + parts[0].length < maxWidth) {
      res[i] += (res[i] === "" ? "" : " ") + parts.shift();
    }
  }
  if(parts.length > 0) {
    res[lines-1] += " " + parts.join(" ");
  }
  return res;
}

function html2svg(str) {
  return str.replace("<u>", "*").replace("</u>", "*").replace(/<span .*>|<\/span>/, '');
}
function formatDate(date) {
  if(date == null)
    return "";
  const d = new Date(date);
  return d.toLocaleDateString();
}